
from __future__ import print_function

import os
import pathlib
import lxml.etree as ET

import osc.core

class Service:
    """
    <services>
      <service name="download_files">
        <param name="enforceupstream">yes</param>
      </service>
    </services>
    """
    def __init__(self, filename):
        """
        @param string filename the filename for the service
        """
        self.__filename = filename

    def init(self):
        """
        Load from disk the services file (or initialize it to empty otherwise).
        @return self
        """
        services = self.__read(self.__filename)

        if services == None:
            services = ET.fromstring('<services/>')

        self.__services = services

        return self

    def write(self):
        """
        Flush local changes to the services file to disk.
        @return self
        """
        with open(self.__filename, 'w') as file:
            file.write(ET.tostring(self.__services, pretty_print=True))

        return self

    def merge(self, filename):
        """
        Merge the contents of the source file into this object.
        @param string sourceFilename the _source file with new service entries
        @return self
        """
        self.__merge(filename)

        return self

    def __read(self, filename):
        """
        @return object the XML object for services or None
        """
        if not os.path.isfile(filename):
            return None

        tree = ET.parse(filename, ET.XMLParser(remove_blank_text=True))

        services = tree.getroot()

        # This is an exception, because I don't want to rewrite the file if it's in an unexpected format.
        assert services.tag == 'services', 'Root tree should begin with services'

        return services

    def __merge(self, filename):
        services = self.__read(filename)

        assert services != None, 'Source _service file does not exist'

        newServices = []
        for service in services:
            s = lambda xml: ET.tostring(xml)

            missing = 0 == len(filter(lambda existing: s(service) == s(existing), self.__services))
            if missing:
                newServices.append(service)

        for newService in newServices:
            self.__services.append(newService)


def run_source_services(service_data, obspkgdir):
    # Parse the service XML data
    try:
        service_xml = ET.fromstring(service_data)
    except ET.ParseError as v:
        line, column = v.position
        print("XML error in source service data on line {}, column {}".format(line, column))
        sys.exit(10)

    # Execute the source services
    svc_info = osc.core.Serviceinfo()
    svc_info.read(service_xml)

    # Service must be run in project dir...
    curdir = os.getcwd()
    os.chdir(obspkgdir)
    svc_exec = svc_info.execute(obspkgdir)
    os.chdir(curdir)
    return svc_exec
